<?php
/**
 * Registers scripts and styles for the blocks.
 *
 * @since 1.0.0
 * @package BlockArt
 */

namespace BlockArt;

/**
 * Registers scripts and styles for the blocks.
 *
 * @since 1.0.0
 */
final class Enqueue_Scripts {

	/**
	 * Cloning is forbidden.
	 *
	 * @since 1.0.0
	 */
	public function __clone() {

		blockart_doing_it_wrong( __FUNCTION__, __( 'You cannot clone this class.', 'blockart' ), '1.0.0' );
	}

	/**
	 * Hooks to register the block scripts and styles.
	 *
	 * @since 1.0.0
	 */
	public static function register() {

		add_action( 'init', array( __CLASS__, 'register_blocks_scripts' ) );
		add_action( 'init', array( __CLASS__, 'register_blocks_styles' ) );
		add_action( 'wp_enqueue_scripts', array( __CLASS__, 'enqueue_block_styles' ) );
	}

	/**
	 * Register script for blocks.
	 *
	 * @since 1.0.0
	 */
	public static function register_blocks_scripts() {

		global $pagenow;
		$asset = self::get_asset_file();

		wp_register_script(
			'blockart-blocks-js',
			plugins_url( '/dist/blocks.js', BLOCKART_PLUGIN_FILE ),
			$asset['dependencies'],
			BLOCKART_VERSION,
			true
		);

		wp_set_script_translations( 'blockart-blocks-js', 'blockart', BLOCKART_LANGUAGES );

		wp_localize_script(
			'blockart-blocks-js',
			'blockart_data',
			array(
				'inline_css'        => 'widgets.php' === $pagenow || 'customize.php' === $pagenow,
				'image_placeholder' => BLOCKART_ASSETS_DIR_URL . '/images/placeholder.png',
			)
		);
	}

	/**
	 * Register all the block styles.
	 *
	 * @since 1.0.0
	 */
	public static function register_blocks_styles() {

		wp_register_style(
			'blockart-blocks-css',
			plugins_url( 'dist/style-blocks.css', BLOCKART_PLUGIN_FILE ),
			is_admin() ? array( 'wp-editor' ) : null,
			BLOCKART_VERSION
		);

		wp_register_style(
			'blockart-blocks-editor-css',
			plugins_url( 'dist/blocks.css', BLOCKART_PLUGIN_FILE ),
			array( 'wp-edit-blocks' ),
			BLOCKART_VERSION
		);
	}

	/**
	 * Add frontend internal style.
	 *
	 * @since 1.0.0
	 */
	public static function add_inline_styles() {

		$post_id = get_the_ID();
		$css     = get_post_meta( $post_id, '_blockart_css', true );

		echo "<style id=\"blockart-post-{$post_id}-css\">{$css}</style>"; // phpcs:ignore
	}

	/**
	 * Enqueue blocks frontend css.
	 *
	 * @since 1.0.0
	 */
	public static function enqueue_block_styles() {

		$post_id                 = get_the_ID();
		$upload_dir_url          = wp_get_upload_dir();
		$blockart_upload_dir_url = trailingslashit( $upload_dir_url['basedir'] );

		if ( $post_id ) {
			$blockart_css_path = $blockart_upload_dir_url . "blockart/blockart-css-{$post_id}.css";

			if ( file_exists( $blockart_css_path ) ) {
				$blockart_css_dir_url = trailingslashit( $upload_dir_url['baseurl'] );

				if ( is_ssl() ) {
					$blockart_css_dir_url = str_replace( 'http://', 'https://', $blockart_css_dir_url );
				}

				$css_url = $blockart_css_dir_url . "blockart/blockart-css-{$post_id}.css";

				wp_enqueue_style( "blockart-post-{$post_id}", $css_url, array(), filemtime( $blockart_css_path ) );
			}
		}

		self::enqueue_google_fonts();
	}

	/**
	 * Enqueue Google fonts.
	 *
	 * @since 1.0.0
	 * @param int $post_id Post ID.
	 */
	public static function enqueue_google_fonts() {

		global $post;

		if ( ! is_object( $post ) ) {
			return;
		}

		$content         = apply_filters( 'blockart_block_content', $post->post_content );
		$blocks          = parse_blocks( $content );
		$font_attributes = self::get_font_attributes( $blocks );
		$google_fonts    = '';

		if ( count( $font_attributes ) > 0 ) {
			foreach ( $font_attributes as $family => $weight ) {
				if ( ! empty( $family ) && 'Default' !== $family ) {
					$google_fonts .= str_replace( ' ', '+', trim( $family ) ) . ':' . join( ',', array_unique( $weight ) ) . '|';
				}
			}

			if ( ! empty( $google_fonts ) ) {
				wp_register_style(
					'blockart-google-fonts',
					add_query_arg(
						array(
							'family'  => $google_fonts,
							'display' => 'swap',
						),
						'//fonts.googleapis.com/css'
					),
					array(),
					BLOCKART_VERSION
				);

				wp_enqueue_style( 'blockart-google-fonts' );
			}
		}
	}

	/**
	 * Get font attributes used in BlockArt blocks.
	 *
	 * @since 1.0.0
	 * @param array $blocks All blocks in a content.
	 * @param array $fonts Used Google fonts.
	 * @return array Font attributes.
	 */
	public static function get_font_attributes( $blocks, $fonts = array() ) {

		foreach ( $blocks as $block ) {
			if ( false !== strpos( $block['blockName'], 'blockart' ) ) {
				foreach ( $block['attrs'] as $key => $attr ) {
					if ( isset( $attr['typography'] ) && isset( $attr['family'] ) ) {
						$weight = $block['attrs'][ $key ]['weight'] ?
								(string) $block['attrs'][ $key ]['weight'] :
								'400';
						$family = $block['attrs'][ $key ]['family'];

						if ( array_key_exists( $family, $fonts ) ) {
							array_push( $fonts[ $family ], $weight );
						} else {
							$fonts[ $family ] = array( $weight );
						}
					}
				}

				if ( isset( $block['innerBlocks'] ) && count( $block['innerBlocks'] ) > 0 ) {
					$inner_block_fonts = self::get_font_attributes( $block['innerBlocks'], $fonts );

					if ( count( $inner_block_fonts ) > 0 ) {
						$fonts = array_merge( $fonts, $inner_block_fonts );
					}
				}
			}
		}

		return $fonts;
	}

	/**
	 * Get asset file generated by wp-scripts.
	 *
	 * @return mixed|string[][] Array.
	 */
	private static function get_asset_file() {

		$asset_file = dirname( BLOCKART_PLUGIN_FILE ) . '/dist/blocks.asset.php';

		if ( ! file_exists( $asset_file ) ) {
			return array( 'dependencies' => array( 'wp-api-fetch', 'wp-block-editor', 'wp-blocks', 'wp-components', 'wp-compose', 'wp-data', 'wp-dom-ready', 'wp-element', 'wp-hooks', 'wp-i18n', 'wp-keyboard-shortcuts' ) );
		}

		return require $asset_file;
	}
}
