<?php
/**
 * BlockArt plugin main class.
 *
 * @since 1.0.0
 * @package BlockArt
 */

namespace BlockArt;

defined( 'ABSPATH' ) || exit;

/**
 * BlockArt setup.
 *
 * Include and initialize necessary files and classes for the plugin.
 *
 * @since   1.0.0
 */
final class Plugin {

	/**
	 * The single instance of the class.
	 *
	 * @since 1.0.0
	 * @var Plugin
	 */
	protected static $instance = null;

	/**
	 * Block Manager.
	 *
	 * Holds the block manager.
	 *
	 * @since 1.0.0
	 * @var Blocks_Manager
	 */
	private $blocks_manager;

	/**
	 * REST API.
	 *
	 * @since 1.0.0
	 * @var REST_API
	 */
	private $rest_api;

	/**
	 * Instance.
	 *
	 * Ensures only instance of Plugin class is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 * @return Plugin - Main instance.
	 */
	public static function instance() {

		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Plugin Constructor.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	private function __construct() {

		add_action( 'init', array( $this, 'init' ), 0 );

		// Register block scripts and styles.
		Enqueue_Scripts::register();

		// Register all blocks and their categories.
		$this->blocks_manager = new Blocks_Manager();

		// Register all custom routes.
		$this->rest_api = new REST_API();
	}

	/**
	 * Initialize BlockArt when WordPress initializes.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function init() {

		// Update the plugin version.
		$this->update_plugin_version();
		$this->load_text_domain();

		/**
		 * BlockArt init.
		 *
		 * Fires after BlockArt has loaded.
		 *
		 * @since 1.0.0
		 */
		do_action( 'blockart_init' );
	}

	/**
	 * Update the plugin version.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	private function update_plugin_version() {

		if ( false === get_option( 'blockart_version' ) ) {
			update_option( 'blockart_version', BLOCKART_VERSION );
		}
	}

	public function load_text_domain() {
		load_plugin_textdomain( 'blockart', false, plugin_basename( dirname( BLOCKART_PLUGIN_FILE ) ) . '/languages' );
	}
}
