<?php
/**
 * Register REST routes.
 *
 * @since 1.0.0
 * @package BlockArt
 */

namespace BlockArt;

use Peast\Syntax\Exception;
use WP_REST_Controller;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;

defined( 'ABSPATH' ) || exit;

/**
 * REST_API class.
 *
 * @since 1.0.0
 */
final class REST_API extends WP_REST_Controller {

	/**
	 * Member variable.
	 *
	 * @since 1.0.0
	 * @var $instance
	 */
	private static $instance;

	/**
	 * The namespace of this controller's route.
	 *
	 * @since 1.0.0
	 * @var string
	 */
	protected $namespace = 'blockart/v1';

	/**
	 * REST API endpoint.
	 *
	 * @since 1.0.0
	 * @var string
	 */
	protected $api_endpoint = 'https://wpblockart.com/wp-json/blockart-library/v1';

	/**
	 * Initiator.
	 *
	 * @since 1.0.0
	 * @return REST_API
	 */
	public static function get_instance() {

		if ( ! isset( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * REST_API constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		add_action( 'rest_api_init', array( $this, 'register_routes' ) );
	}

	/**
	 * Register REST routes.
	 *
	 * @since 1.0.0
	 */
	public function register_routes() {

		register_rest_route(
			$this->namespace,
			'/save_block_css/',
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'save_block_css' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route(
			$this->namespace,
			'/get_block_content/',
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'get_block_content' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route(
			$this->namespace,
			'/save_reusable_block_css/',
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => array( $this, 'append_block_css' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route(
			$this->namespace,
			'/get_section/',
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => function( $request ) {
					return $this->get_designs( $request );
				},
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route(
			$this->namespace,
			'/get_single_section/',
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => function( $request ) {
					return $this->get_content( $request );
				},
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route(
			$this->namespace,
			'/get_template/',
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => function( $request ) {
					return $this->get_designs( $request, 'templates' );
				},
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route(
			$this->namespace,
			'/get_single_template/',
			array(
				'methods'             => WP_REST_Server::EDITABLE,
				'callback'            => function( $request ) {
					return $this->get_content( $request, 'single-template' );
				},
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);
	}

	/**
	 * Create files.
	 *
	 * @since 1.0.0
	 * @param string $filename Filename.
	 * @param string $content Content.
	 * @throws \Exception
	 * @return void|WP_REST_Response
	 */
	public function create_files( $filename = '', $content = '' ) {

		try {
			global $wp_filesystem;

			$upload_dir_url = wp_upload_dir();
			$dir            = trailingslashit( $upload_dir_url['basedir'] ) . 'blockart/';

			if ( ! $wp_filesystem ) {
				require_once ABSPATH . 'wp-admin/includes/file.php';
			}

			WP_Filesystem( false, $upload_dir_url['basedir'], true );

			if ( ! $wp_filesystem->is_dir( $dir ) ) {
				$wp_filesystem->mkdir( $dir );
			}

			if ( ! $wp_filesystem->put_contents( $dir . $filename, $content ) ) {
				throw new \Exception( __( 'Permission failed to save file!', 'blockart' ) );
			}
		} catch ( \Exception $e ) {
			return new WP_REST_Response(
				array(
					'success' => false,
					'message' => $e->getMessage(),
				)
			);
		}
	}

	/**
	 * Save block CSS in post meta and on a file.
	 *
	 * @since 1.0.0
	 * @param WP_REST_Request $request Data about the request.
	 * @return WP_REST_Response
	 * @throws \Exception
	 */
	public function save_block_css( $request ) {

		$params         = $request->get_params();
		$post_id        = (int) sanitize_text_field( $params['postId'] );
		$filename       = "blockart-css-{$post_id}.css";
		$upload_dir_url = wp_upload_dir();
		$dir            = trailingslashit( $upload_dir_url['basedir'] ) . 'blockart/';

		if ( $params['has_block'] ) {
			$this->create_files( $filename, $params['blockCss'] );
			update_post_meta( $post_id, '_blockart_active', 'yes' );
			update_post_meta( $post_id, '_blockart_css', $params['blockCss'] );
		} else {
			delete_post_meta( $post_id, '_blockart_active' );

			if ( file_exists( $dir . $filename ) ) {
				unlink( $dir . $filename );
			}

			delete_post_meta( $post_id, '_blockart_css' );
		}

		return new WP_REST_Response(
			array(
				'success' => true,
				'message' => __( 'Data retrieved successfully.', 'blockart' ),
			)
		);
	}

	/**
	 * Get post content.
	 *
	 * To handle the reusable blocks as WordPress treats reusable block as a separate post type.
	 *
	 * @since 1.0.0
	 * @param WP_REST_Request $request Data about the request.
	 * @return WP_REST_Response
	 */
	public function get_block_content( $request ) {

		$post = $request->get_params();

		if ( isset( $post['postId'] ) ) {
			return new WP_REST_Response(
				array(
					'success' => true,
					'data'    => get_post( $post['postId'] )->post_content,
					'message' => __( 'Data retrieved successfully.', 'blockart' ),
				)
			);
		} else {
			return new WP_REST_Response(
				array(
					'success' => false,
					'message' => __( 'Failed to retrieve data!', 'blockart' ),
				)
			);
		}
	}

	/**
	 * Append/Create css of the reusable blocks.
	 *
	 * @since 1.0.0
	 * @param WP_REST_Request $request Data about the request.
	 * @throws \Exception
	 * @return WP_REST_Response
	 */
	public function append_block_css( $request ) {

		$post    = $request->get_params();
		$css     = $post['css'];
		$post_id = (int) sanitize_text_field( $post['postId'] );

		if ( $post_id ) {
			global $wp_filesystem;

			if ( ! $wp_filesystem ) {
				require_once ABSPATH . 'wp-admin/includes/file.php';
			}

			$filename = "blockart-css-{$post_id}.css";
			$dir      = trailingslashit( wp_upload_dir()['basedir'] ) . 'blockart/';

			if ( file_exists( $dir . $filename ) ) {

				$get_data = get_post_meta( $post_id, '_blockart_css', true );

				if ( false === strpos( $get_data, $css ) ) {
					// phpcs:disable
					$file = fopen( $dir . $filename, 'a' );
					fwrite( $file, $css );
					fclose( $file );
					// phpcs:enable

					update_post_meta( $post_id, '_blockart_css', $get_data . $css );
				}
			} else {
				$this->create_files( $filename, $css );
				update_post_meta( $post_id, '_blockart_active', 'yes' );
				update_post_meta( $post_id, '_blockart_css', $css );
			}

			return new WP_REST_Response(
				array(
					'success' => true,
					'message' => __( 'Data retrieved successfully.', 'blockart' ),
				)
			);
		}

		return new WP_REST_Response(
			array(
				'success' => false,
				'message' => __( 'Failed to retrieve data!', 'blockart' ),
			)
		);
	}

	/**
	 * Fetch pre-made designs using self-hosted api endpoint.
	 *
	 * @since 1.0.0
	 * @param WP_REST_Request $request Data about the request.
	 * @param string $type Type of data to fetch.
	 * @return WP_REST_Response
	 */
	public function get_designs( $request, $type = 'sections' ) {

		$param = $request->get_params();

		$remote_data = wp_remote_post(
			$this->api_endpoint . "/{$type}",
			array(
				'method'  => 'POST',
				'timeout' => 120,
				'body'    => array(
					'starter_pack' => isset( $param['starter_pack'] ) && (bool) $param['starter_pack'],
				),
			)
		);

		if ( is_wp_error( $remote_data ) ) {
			return new WP_REST_Response(
				array(
					'success' => false,
					'message' => __( 'Failed to retrieve data!', 'blockart' ),
				)
			);
		}

		return new WP_REST_Response(
			array(
				'success' => true,
				'message' => __( 'Data retrieved successfully.', 'blockart' ),
				'data'    => $remote_data['body'],
			)
		);
	}

	/**
	 * Fetch content using self-hosted api endpoint.
	 *
	 * @since 1.0.0
	 * @param WP_REST_Request $request Data about the request.
	 * @param string $type Type of data to fetch.
	 * @return WP_REST_Response
	 */
	public function get_content( $request, $type = 'single-section' ) {

		$params      = $request->get_params();
		$remote_data = wp_remote_post(
			$this->api_endpoint . "/{$type}",
			array(
				'method'  => 'POST',
				'timeout' => 120,
				'body'    => array(
					'id' => $params['id'],
				),
			)
		);

		if ( is_wp_error( $remote_data ) ) {
			return new WP_REST_Response(
				array(
					'success' => false,
					'message' => __( 'Failed to retrieve data!', 'blockart' ),
				)
			);
		}

		$data              = json_decode( $remote_data['body'] );
		$processed_content = $this->process_content( $data->content );

		return new WP_REST_Response(
			array(
				'success' => true,
				'message' => __( 'Data retrieved successfully.', 'blockart' ),
				'data'    => $processed_content,
			)
		);
	}

	/**
	 * Process content while importing section or template.
	 *
	 * @since 1.0.0
	 * @param string $content Post content.
	 * @return array|string|string[]
	 */
	public function process_content( $content = '' ) {

		preg_match_all( '#\bhttps?://[^,\s()<>]+(?:\([\w\d]+\)|([^,[:punct:]\s]|/))#', $content, $match );

		$urls = array_unique( $match[0] );

		if ( empty( $urls ) ) {
			return $content;
		}

		$map_urls   = array();
		$image_urls = array();

		foreach ( $urls as $key => $url ) {
			if ( $this->has_image( $url ) ) {
				if (
					false === strpos( $url, '-150x' ) &&
					false === strpos( $url, '-300x' ) &&
					false === strpos( $url, '-1024x' )
				) {
					$image_urls[] = $url;
				}
			}
		}

		if ( ! empty( $image_urls ) ) {
			foreach ( $image_urls as $key => $image_url ) {
				$image                  = array(
					'url' => $image_url,
					'id'  => 0,
				);
				$downloaded_image       = $this->import_image( $image );
				$map_urls[ $image_url ] = $downloaded_image['url'];
			}
		}

		foreach ( $map_urls as $old_url => $new_url ) {
			$content = str_replace( $old_url, $new_url, $content );
			$old_url = str_replace( '/', '/\\', $old_url );
			$new_url = str_replace( '/', '/\\', $new_url );
			$content = str_replace( $old_url, $new_url, $content );
		}

		return $content;
	}

	/**
	 * Check if the URL points to an image.
	 *
	 * @since 1.0.0
	 * @param string $url Valid URL.
	 * @return false|int
	 */
	public function has_image( $url = '' ) {
		return preg_match( '/^((https?:\/\/)|(www\.))([a-z0-9-].?)+(:[0-9]+)?\/[\w\-]+\.(jpg|png|gif|jpeg)\/?$/i', $url );
	}

	/**
	 * Handles image import.
	 *
	 * @since 1.0.0
	 * @param array $data Image URL data.
	 * @return array|mixed
	 */
	public function import_image( $data ) {

		$local_image = $this->is_local_image( $data );

		if ( $local_image['status'] ) {
			return $local_image['image'];
		}

		$file_content = wp_remote_retrieve_body(
			wp_safe_remote_get(
				$data['url'],
				array(
					'timeout'   => '60',
					'sslverify' => false,
				)
			)
		);

		if ( empty( $file_content ) ) {
			return $data;
		}

		$filename = basename( $data['url'] );

		$upload = wp_upload_bits( $filename, null, $file_content );
		$post   = array(
			'post_title' => $filename,
			'guid'       => $upload['url'],
		);
		$info   = wp_check_filetype( $upload['file'] );

		if ( $info ) {
			$post['post_mime_type'] = $info['type'];
		} else {
			return $data;
		}

		$post_id = wp_insert_attachment( $post, $upload['file'] );

		require_once ABSPATH . 'wp-admin/includes/image.php';

		wp_update_attachment_metadata(
			$post_id,
			wp_generate_attachment_metadata( $post_id, $upload['file'] )
		);
		update_post_meta( $post_id, '_blockart_image_hash', sha1( $data['url'] ) );

		return array(
			'id'  => $post_id,
			'url' => $upload['url'],
		);
	}

	/**
	 * Check if the image exists in the system.
	 *
	 * @since 1.0.0
	 * @param array $data Image URL data.
	 * @return array
	 */
	public function is_local_image( $data ) {

		global $wpdb;

		$image_id = $wpdb->get_var(
			$wpdb->prepare(
				'SELECT `post_id` FROM `' . $wpdb->postmeta . '`
					WHERE `meta_key` = \'_blockart_image_hash\'
						AND `meta_value` = %s
				;',
				sha1( $data['url'] )
			)
		);

		if ( $image_id ) {
			$local_image = array(
				'id'  => $image_id,
				'url' => wp_get_attachment_url( $image_id ),
			);
			return array(
				'status' => true,
				'image'  => $local_image,
			);
		}
		return array(
			'status' => false,
			'image'  => $data,
		);
	}

	/**
	 * Permission check callback.
	 *
	 * @since 1.0.0
	 * @return bool
	 */
	public function permission_check() {
		return current_user_can( 'edit_posts' );
	}
}
